#define HARDRWARE_SPI
#include <stdint.h>
#include <stdio.h>
#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "rfm.h"

typedef unsigned char u8;
typedef unsigned int u16;
typedef unsigned long u32;

#define output_low(port,pin) port &= ~(1<<pin)
#define output_high(port,pin) port |= (1<<pin)
#define set_input(portdir,pin) portdir &= ~(1<<pin)
#define set_output(portdir,pin) portdir |= (1<<pin)

#define LED1OFF          output_low(PORTB,1)
#define LED1ON           output_high(PORTB,1)

#define LED2OFF          output_low(PORTB,2)
#define LED2ON           output_high(PORTB,2)

#define int0_off EIMSK=0x00;
#define int0_on EIMSK=0x01;

u8 * buff;
u8 b1;
u8 b2;
u8 i;
u8 data1[8];
u8 data2[8];
u8 ovf;
volatile u8 idx;



void TxByte (u8 data)
{
   while ( !( UCSR0A & (1<<UDRE0)) );              
   UDR0 = data;                                   
}

u8 RxByte (void)
{
   while ( !(UCSR0A & (1<<RXC0)) );                
   return UDR0;                                   
} 

ISR(INT0_vect) {
	LED1ON;
	cli();
	b1=GetRFMByte();

	if(b1==1){
		buff=GetRcvdData();
		TxByte('\n');
		TxByte('\r');
		for(i=0;i<7;i++){
        	b1=buff[i];
			TxByte(b1);
    	}
	}

	sei();
	LED1OFF;
}

void sendPreparedData(){
	int0_off;
	rfTxMode();
	SendPackage(data1);
	rfRxMode();
	FIFOReset();
	int0_on;	
}

ISR(USART0_RX_vect){
	//asm("jmp 0x7000");
	if(!ovf){
		data1[idx]=RxByte();
		idx=idx+1;
		if(idx>6){
			idx=0;
			ovf=1;
			data1[7]=GetCRC(data1);
		}
	}	

	if(ovf==1){
		LED2ON;
		sendPreparedData();
		TxByte('\n');
		TxByte('\r');
		TxByte('+');
		TxByte('\n');
		TxByte('\r');
		LED2OFF;
		ovf=0;
	}
}

int init(){
	// Port B - all out
	PORTB=0x00;
	DDRB=0xFF;


	// INT0 Mode: Low level
	EICRA=0x00;
	EIMSK=0x01;
	EIFR=0x01;
	PCICR=0x00;

	// USART0: 8N1, Rx, Tx, Async, 115200
	UCSR0A=0x00;
	UCSR0B=0x18;
	UCSR0C=0x06;
	UBRR0H=0x00;
	UBRR0L=0x05;
	ovf=0;

	//UART interrupts
	UCSR0B=(1<<RXEN0)|(1<<TXEN0)|(1<<RXCIE0);

	initRFMPort();
	rfInit();
	rfRxMode();
	FIFOReset();
	return 0;
}
int main(){
	init();
	TxByte('h');
	TxByte('e');
	TxByte('l');
	TxByte('l');
	TxByte('o');
	TxByte('\n');
	TxByte('\r');
	sei();
	while(1){
	}
	return 0;
}
