#include <avr/io.h>
#include <avr/eeprom.h>
#include "rfm.h"

unsigned char buf[20];
unsigned char pos=0, err=0, rcvd=0;
unsigned char uartbufOut[18];

unsigned char comlen = 8;

void setCommandLength(unsigned char len)
{
	comlen = len;
}

void initRFMPort()
{
#ifndef HARDWARE_SPI
	RFM_SCK_DDR |= (1 << RFM_SCK_BIT);
	RFM_SDO_DDR &= ~(1 << RFM_SDO_BIT);
	RFM_SDI_DDR |= (1 << RFM_SDI_BIT);
	RFM_CS_DDR  |= (1 << RFM_CS_BIT);
	
#else

	DDR_SPI = (1<<DD_MOSI)|(1<<DD_SCK)|(1<<DD_SS);
	SPCR0 = (1<<SPE0)|(1<<MSTR0);
#endif
	NIRQ_DDR &= ~(1 << NIRQ_BIT);
}

unsigned int writeCmd(unsigned int cmd) 
{
unsigned int recv;
#ifndef HARDWARE_SPI

	unsigned char i;
  	
  	recv = 0;
  	
	RFM_CS_PORT &= ~(1 << RFM_CS_BIT);
  	for(i=0; i<16; i++) 
	{
    	if(cmd&0x8000) 
			RFM_SDI_PORT |= (1 << RFM_SDI_BIT);
    	else 
			RFM_SDI_PORT &= ~(1 << RFM_SDI_BIT);
		RFM_SCK_PORT |= (1 << RFM_SCK_BIT);
    	recv<<=1;
    	if(RFM_SDO_PIN & (1 << RFM_SDO_BIT) ) 
		{
      		recv|=0x0001;
	    }
		RFM_SCK_PORT &= ~(1 << RFM_SCK_BIT);
     	cmd<<=1;
	}
	RFM_CS_PORT |= (1 << RFM_CS_BIT);
  	
#else
	PORT_SPI &= ~(1 << DD_SS);
	SPDR0 = cmd >> 8;
	while(!(SPSR0 & (1<<SPIF0)));
	recv = SPDR0 << 8;
	
	SPDR0 = cmd & 0xff;
	while(!(SPSR0 & (1<<SPIF0)));

	recv |= SPDR0;
	PORT_SPI |= (1 << DD_SS);
#endif
	return recv;
}

void FIFOReset() 
{
	writeCmd(0xCA81);
  	writeCmd(0xCA83);
}

void rfInit() 
{
	//writeCmd(0x80D8); //EL,EF,433band,12.5pF
	writeCmd(0x80E7); //868
 	writeCmd(0xA640); //frequency select
	writeCmd(0x8201);
  	//writeCmd(0xC608); //38.4kbps
	writeCmd(0xC647); //4.8kbps
  	writeCmd(0x94A0); //VDI,FAST,134kHz,0dBm,-103dBm
  	writeCmd(0xC2AC); //AL,!ml,DIG,DQD4
  	writeCmd(0xCA81); //FIFO8,SYNC,!ff,DR
  	writeCmd(0xCED4); //SYNC=2DD4G
  	writeCmd(0xC483); //@PWR,NO RSTRIC,!st,!fi,OE,EN
  	writeCmd(0x9850); //!mp,90kHz,MAX OUT
  	writeCmd(0xCC17); //OB1COB0, LPX,IddyCDDITCBW0
  	writeCmd(0xE000); //NOT USE
  	writeCmd(0xC800); //NOT USE
  	writeCmd(0xC040); //1.66MHz,2.2V
}

void rfTxMode()
{
    writeCmd(0x8239); 
}

void rfRxMode()
{
    writeCmd(0x8299); 
}

unsigned char rfRecv() 
{
	unsigned int t = 0;
	unsigned int data;
	while(1) 
	{
		t++;
    	data = writeCmd(0x0000);
    	if (data & 0x8000) 
		{
      		data = writeCmd(0xB000);
      		return (data&0x00FF);
    	}
		if (t == 60000)
			return 0;
  	}  
}

void rfSend(unsigned char data)
{
	while (WAIT_NIRQ_LOW()) {};
  	writeCmd(0xB800 + data);
}

void SendByte(unsigned char data)
{
    writeCmd(0x0000);        
    
    rfSend(0xAA); // PREAMBLE
    rfSend(0xAA);
    rfSend(0xAA);
    rfSend(0x2D); // SYNC
    rfSend(0xD4);    
    rfSend(data);
    rfSend(0xAA); // DUMMY BYTES
    rfSend(0xAA);
    rfSend(0xAA);    
}

void SendPackage(unsigned char* data)
{
    unsigned char i;
    writeCmd(0x0000);        
    
    rfSend(0xAA); // PREAMBLE
    rfSend(0xAA);
    rfSend(0xAA);
    rfSend(0x2D); // SYNC
    rfSend(0xD4);    
    for(i=0;i<comlen;i++)
    {
        rfSend(*data);
        data++;
    }
    rfSend(0xAA); // DUMMY BYTES
    rfSend(0xAA);
    rfSend(0xAA);    
}

void sendPack(unsigned char s[8]){
	unsigned int i;
	char crc=0;
	rfTxMode();
	for(i=0;i<8;i++){
		SendByte(s[i]);
		crc^=s[i];
	}
	SendByte(crc);
	rfRxMode();
}

unsigned char GetCRC(unsigned char* data)
{
    unsigned char CRC=0, i;
    for(i=0;i<comlen-1;i++)
    {
        CRC^=(*data);
        data++;
    }     
    return CRC;
}

unsigned char GetRFMByte()
{
    unsigned char GotFullPack = 0;
    buf[pos] = rfRecv();
    pos++;
    if(pos==comlen)   
    {
        if(GetCRC(buf)==buf[comlen-1])
            GotFullPack = 1; 
        else
            GotFullPack = 2;   
        pos=0;         
        FIFOReset();
    }   
    return GotFullPack;
} 

unsigned char* GetRcvdData()
{
    return buf;
}

unsigned char* PrepareUartData(unsigned char* data)
{
	unsigned int i;
	uartbufOut[0] = 0xAA;
	uartbufOut[1] = comlen;
	for(i = 0; i < comlen; i++)
		uartbufOut[i + 2] = *(data + i);
	return uartbufOut;
}


void Answer(unsigned char* data)
{
	unsigned char t;
	t = *(data);
	*(data) = *(data + 2);
	*(data + 2) = t;
	t = *(data + 1);
	*(data + 1) = *(data + 3);
	*(data + 3) = t;
	SendPackage(data);
}

void rfmSleep()
{
	writeCmd(0x8201);
}



