/*
 * terrarium.c
 *
 * Created: 05.09.2011 21:38:29
 *  Author: vsd
 */ 
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include "therm_ds18b20.h"
#include "display.h"
#include "ds1307.h"
#include "rfm.h"

#define TEMP_BUFFER_SIZE 10
#define DAY_MODE 1
#define NIGHT_MODE 2

#define BUTTON_PUSH_LIMIT 5 //      ,      
#define BUTTON_FAST_LIMIT 300 //    

#define BUTTON_OFF 0
#define BUTTON_PRESSED 1
#define BUTTON_FAST 2

#define MODE_EDITING 1
#define MODE_MENU 0

#define heater_on bit_set(PORTD,_BV(PD6))
#define heater_off bit_clear(PORTD,_BV(PD6))

#define int1_off GICR&=~(1<<INT1)
#define int1_on GICR|=(1<<INT1)


uint8_t currentDayTime;
uint8_t  * buff={0,0,0,0,0,0,0,0};
volatile uint8_t needToSendPackage;
	
uint8_t d[4]={12,17,20,0};//HELO -    
uint8_t points[4]={0,0,0,0};
uint16_t ticker=0;
uint16_t led_steps=0;
uint16_t led_current_step=0;
uint8_t current_mode=MODE_MENU;
uint8_t old_mode;
uint16_t old_temp;
uint8_t currentColor[3]={0,0,0};//R,G,B
volatile uint8_t backToMenuCounter=0;

//D1,D2,D3,D4,start_position,length,minus,clear_other,dot_position,min,max
uint8_t menu[24][11]={// . 98 -   -, 99     , 110 -  
{99,99,99,99 ,0,3,0,0,1,0,255},//  ,   (/).     -   :  -  1,  -  2
{10,1,5,99   ,0,3,0,1,1,0,255},//     1. : 15-40,  0.1.  - t1S
{10,1,12,99  ,0,2,0,1,0,1,99}, //     1. : 0.1-9.9,  0.1.  - t1H
{10,1,13,13  ,0,2,0,1,5,0,23}, //     1. : 00-23,  1.  - t1hh
{10,1,14,14  ,0,2,0,1,5,0,59}, //     1. : 00-59,  1.  - t1nn
{10,2,5,110  ,0,3,0,1,1,0,255},//     2. : 15-40,  0.1.  - t2S
{10,2,12,99  ,0,2,0,1,0,1,99}, //     2. : 0.1-9.9,  0.1.  - t2H
{10,2,13,13  ,0,2,0,1,5,0,23}, //     2. : 00-23,  1.  - t2hh
{10,2,14,14  ,0,2,0,1,5,0,59}, //     2. : 00-59,  1.  - t2nn
{17,98,99,99,1,2,1,0,2,28,226},//  . : +/-9.9,  0.1.  - E
{99,99,110,13,0,2,0,0,5,0,23}, //  . : 00-23,  1.   h
{99,99,14,14 ,0,2,0,0,5,0,59}, //  . : 00-59,  1.  nn
{99,99,110,5 ,0,2,0,0,5,0,59}, //  . : 00-59,  1.   S
{99,99,24,1  ,0,2,0,0,5,0,31}, //  . : 00-31,  1.   d1
{99,99,24,2  ,0,2,0,0,5,0,12}, //  . : 00-12,  1.  d2
{99,99,24,3  ,0,2,0,0,5,11,99},//  . : 11-99,  1.   d3
{18,99,98,99,1,2,1,0,5,28,226},//    .      : +/-99,  1.  - C
{19,17,24,1  ,0,3,0,1,5,0,255},//       1. : 000-255. : rEd1
{22,19,17,1  ,0,3,0,1,5,0,255},//       1. : 000-255. : GrE1
{23,20,21,1  ,0,3,0,1,5,0,255},//       1. : 000-255. : blu1
{19,17,24,2  ,0,3,0,1,5,0,255},//       2. : 000-255. : rEd2
{22,19,17,2  ,0,3,0,1,5,0,255},//       2. : 000-255. : GrE2
{23,20,21,2  ,0,3,0,1,5,0,255},//       2. : 000-255. : blu2
{10,20,1,99  ,0,3,0,1,5,0,255}};//     . : 000-255 . : tLI
	
uint8_t data[24]={250,100,10,06,00,80,10,18,00,107,00,00,00,01,01,12,127,255,200,220,25,20,20,30};
//      -    15 . 
//    RAM      uint_16    (    )

uint16_t target_temp[2];//  (/)

volatile uint8_t interrupt_triggered=0;//   
uint8_t cdd=0;//  
uint8_t btnState[3];//, , set -  
uint16_t btnTimer[3];//, , set -  
uint8_t currentMenuItem=0;//  

uint16_t currentTemp;//  * 10

uint16_t temp_buffer[TEMP_BUFFER_SIZE];



unsigned char data1[10];//  RFM

//  - 1       
ISR(INT0_vect){
	interrupt_triggered=1;
}

//RFM interrupt
ISR(INT1_vect){
	unsigned char b1;
	b1=GetRFMByte();

	uint8_t flag=0;
	uint8_t position=0;
	if(b1==1){
		buff=GetRcvdData();
		if(buff[0]=='T'){// ,   T
			if(buff[1]=='G'){//   - G -     
				data1[0]='C';//C - Confirm
				data1[1]='G';
				data1[2]=buff[2];
				data1[3]=buff[3];
				position=(buff[2]-0x30)*10+(buff[3]-0x30);
				if(position<24){
					data1[4]=data[position]/100;
					data1[5]=data[position]/10-data1[4]*10+0x30;
					data1[6]=data[position]%10+0x30;
					data1[4]+=0x30;
				}else if(position<27){//     (    data)
					data1[4]=currentColor[position-24]/100;
					data1[5]=currentColor[position-24]/10-data1[4]*10+0x30;
					data1[6]=currentColor[position-24]%10+0x30;
					data1[4]+=0x30;
				}else{	
					data1[0]='E';//E - error -   ,     
					data1[4]=buff[4];
					data1[5]=buff[5];
					data1[6]=buff[6];
				}
				flag=1;
			}else if(buff[1]=='S'){//S -    
				data1[0]='C';
				data1[1]='S';
				data1[2]=buff[2];
				data1[3]=buff[3];
				data1[4]=buff[4];
				data1[5]=buff[5];
				data1[6]=buff[6];
				position=(buff[2]-0x30)*10+(buff[3]-0x30);
				if(position<24){
					uint8_t dataToSet=0;
					//,   
					dataToSet=(data1[4]-0x30)*100+(data1[5]-0x30)*10+(data1[6]-0x30);					
					//   
					if(dataToSet>menu[position][10]||dataToSet<menu[position][9]){
						data1[0]='E';//E - error -     -      
					}else{
						data[position]=dataToSet;
						if(position>9&&position<16){
							rtc_set_time(data[10],data[11],data[12]);
							rtc_set_date(data[13],data[14],data[15]);
						}else{
							recalculate_target();
							rtc_set_byte(data[position],position+8);
						}
					}					
				}else{
					data1[0]='E';//E - error -   ,   
				}
				flag=1;
			}
		}
	}
	if(flag){
		needToSendPackage=1;
	}
}

void recalculate_target(){
	target_temp[0]=data[1]+150;
	target_temp[1]=data[5]+150;
}

void process_button(uint8_t index,uint8_t state){
	if(state){
		btnState[index]=BUTTON_OFF;
		btnTimer[index]=0;		
	}else{
		btnTimer[index]+=1;
		if(btnTimer[index]>BUTTON_PUSH_LIMIT){
			if(btnTimer[index]>BUTTON_FAST_LIMIT){
				btnState[index]=BUTTON_FAST;
				btnTimer[index]-=1;
			}else{
				if(btnTimer[index]==(BUTTON_PUSH_LIMIT*2)){
					btnState[index]=BUTTON_PRESSED;
				}else{
					btnState[index]=BUTTON_OFF;
				}				
			}
		}
	}		
}

uint8_t can_act(uint8_t index,uint8_t repeat){
	if(btnState[index]!=BUTTON_OFF){
		if(btnState[index]!=BUTTON_FAST||repeat){
			return 1;
		}
	}
	return 0;	
}

void updateDisplayInfo(){
	if(current_mode==MODE_MENU){
		d[0]=menu[currentMenuItem][0];
		d[1]=menu[currentMenuItem][1];
		d[2]=menu[currentMenuItem][2];
		d[3]=menu[currentMenuItem][3];
		points[0]=0;
		points[1]=0;
		points[2]=0;
		if(menu[currentMenuItem][7]==0){
			valueToDisplay(data[currentMenuItem],menu[currentMenuItem][5],menu[currentMenuItem][4],menu[currentMenuItem][6],menu[currentMenuItem][7],menu[currentMenuItem][8]);
		}
	}else{
		if(currentMenuItem!=1&&currentMenuItem!=5){
			valueToDisplay(data[currentMenuItem],menu[currentMenuItem][5],menu[currentMenuItem][4],menu[currentMenuItem][6],menu[currentMenuItem][7],menu[currentMenuItem][8]);
		}else{
			recalculate_target();
			valueToDisplay(data[currentMenuItem]+150,menu[currentMenuItem][5],menu[currentMenuItem][4],menu[currentMenuItem][6],menu[currentMenuItem][7],menu[currentMenuItem][8]);
		}
	}	
	if(currentMenuItem==0){
		valueToDisplay(currentTemp,3,0,0,1,1);
		d[3]=15+(currentDayTime==NIGHT_MODE);
	}
}

//digits switch and keys processing
ISR(TIMER2_OVF_vect){
	digit(cdd,d[cdd],points[cdd]);
	cdd=(cdd+1)%4;
	ticker=(ticker+1);
	uint8_t repeat=0;
	if(ticker%55==0){
		repeat=1;
	}
	process_button(0,((PIND & _BV(PD7))));//up
	process_button(1,((PINC & _BV(PC7))));//down
	process_button(2,((PINC & _BV(PC6))));//set
	if(can_act(2,0)){
		backToMenuCounter=0;//  
		if(current_mode==MODE_MENU){
			current_mode=MODE_EDITING;
		}else{
			current_mode=MODE_MENU;
		}
		updateDisplayInfo();
	}
	if((can_act(0,repeat)||can_act(1,repeat))){
		backToMenuCounter=0;//  
		if(current_mode==MODE_MENU){
			currentMenuItem=(currentMenuItem+can_act(1,repeat))%24;
			if(can_act(0,repeat)){
				if(currentMenuItem==0){
					currentMenuItem=23;
				}else{
					currentMenuItem=currentMenuItem-1;
				}
			}
			updateDisplayInfo();
		}else{
			if(currentMenuItem>0){
				if(data[currentMenuItem]<menu[currentMenuItem][10]){
					data[currentMenuItem]+=can_act(0,repeat);
				}else{
					if(can_act(0,repeat)){
						data[currentMenuItem]=menu[currentMenuItem][9];
					}
				}
				if(data[currentMenuItem]>menu[currentMenuItem][9]){
					data[currentMenuItem]-=can_act(1,repeat);
				}else{
					if(can_act(1,repeat)){
						data[currentMenuItem]=menu[currentMenuItem][10];
					}
				}
			}
			rtc_set_byte(data[currentMenuItem],currentMenuItem+8);
			updateDisplayInfo();
		}		
	}
	TCNT2=0xF0;
}

void pushTemp(uint16_t temp){
	uint8_t i=0;
	uint16_t sum_total=0;
	for(i=TEMP_BUFFER_SIZE-1;i>0;i--){
		temp_buffer[i]=temp_buffer[i-1];
		sum_total+=temp_buffer[i];
	}		
	temp_buffer[0]=(temp+data[9]-127)>0?(temp+data[9]-127):0;
	sum_total+=temp_buffer[0];
	currentTemp=sum_total/(TEMP_BUFFER_SIZE);
}


void init(){


	// Input/Output Ports initialization
	// Port A initialization
	// Func7=Out Func6=Out Func5=Out Func4=Out Func3=Out Func2=Out Func1=Out Func0=Out 
	// State7=1 State6=1 State5=1 State4=1 State3=1 State2=1 State1=1 State0=1 
	PORTA=0xFF;
	DDRA=0xFF;

	// Port B initialization
	// Func7=Out Func6=Out Func5=Out Func4=Out Func3=Out Func2=Out Func1=Out Func0=Out 
	// State7=0 State6=0 State5=0 State4=0 State3=0 State2=0 State1=0 State0=0 
	PORTB=0x00;
	DDRB=0xFF;

	// Port C initialization
	// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
	// State7=P State6=P State5=T State4=T State3=T State2=T State1=T State0=T 
	PORTC=0xC0;
	DDRC=0x00;

	// Port D initialization
	// Func7=In Func6=Out Func5=Out Func4=Out Func3=In Func2=In Func1=Out Func0=Out 
	// State7=P State6=0 State5=0 State4=0 State3=P State2=P State1=0 State0=0 
	PORTD=0x8C;
	DDRD=0x73;


	// External Interrupt(s) initialization
	// INT0: On
	// INT0 Mode: Falling Edge
	// INT1: On
	// INT1 Mode: Low level
	// INT2: Off
	GICR|=0xC0;
	MCUCR=0x02;
	MCUCSR=0x00;
	GIFR=0xC0;
	
	

	// Analog Comparator initialization
	// Analog Comparator: Off
	// Analog Comparator Input Capture by Timer/Counter 1: Off
	ACSR=0x80;
	SFIOR=0x00;

	// Timer/Counter 0 initialization
	// Clock source: System Clock
	// Clock value: 31,250 kHz
	// Mode: Fast PWM top=0xFF
	// OC0 output: Inverted PWM
	TCCR0=0x7C;
	TCNT0=0x00;
	OCR0=0x00;

	// Timer/Counter 1 initialization
	// Clock source: System Clock
	// Clock value: 31,250 kHz
	// Mode: Fast PWM top=0x00FF
	// OC1A output: Inverted
	// OC1B output: Inverted
	// Noise Canceler: Off
	// Input Capture on Falling Edge
	// Timer1 Overflow Interrupt: Off
	// Input Capture Interrupt: Off
	// Compare A Match Interrupt: Off
	// Compare B Match Interrupt: Off
	TCCR1A=0xF1;
	TCCR1B=0x0C;
	TCNT1H=0x00;
	TCNT1L=0x00;
	ICR1H=0x00;
	ICR1L=0x00;
	OCR1AH=0x00;
	OCR1AL=0x00;
	OCR1BH=0x00;
	OCR1BL=0x00;
	
	// Timer/Counter 2 initialization
	// Clock source: System Clock
	// Clock value: 7,813 kHz
	// Mode: Normal top=0xFF
	// OC2 output: Disconnected
	ASSR=0x00;
	TCCR2=0x07;
	TCNT2=0x00;
	OCR2=0x00;

	// Timer(s)/Counter(s) Interrupt(s) initialization
	TIMSK=0x40;
	
	uint8_t check_byte=0;// 50-      99,  ,       .      ,     
	uint8_t i=0;
	rtc_get_byte(&check_byte,50);
	if(check_byte==99){
		for(i=0;i<24;i++){//    
			rtc_get_byte(&data[i],i+8);
		}
	}else{
		rtc_set_byte(99,50);
		for(i=0;i<24;i++){//      
			rtc_set_byte(data[i],i+8);
		}
	}	
	
	recalculate_target();//    
	
	currentDayTime=DAY_MODE;
	rtc_init(0,1,0);
	initRFMPort();
	rfInit();
	rfRxMode();
	FIFOReset();
	

	data1[0]='H';
	data1[1]='E';
	data1[2]='L';
	data1[3]='L';
	data1[4]='O';
	data1[5]='!';
	data1[6]=' ';
	data1[7]=GetCRC(data1);
	rfTxMode();
	SendPackage(data1);
	rfRxMode();
	FIFOReset();
	
	sei();

	//   
	for(i=0;i<TEMP_BUFFER_SIZE;i++){
		_delay_ms(100);
		pushTemp(therm_read_temperature());
	}
	
}
					// , ,  , ?, ? (5- )
void valueToDisplay(int16_t value,uint8_t length,uint8_t start_pos,uint8_t minus,uint8_t clear_other,uint8_t dot_position){
	if(clear_other){
		d[0]=110;
		d[1]=110;
		d[2]=110;
		d[3]=110;
	}
	if(minus){
		if((value-127)<0){//   -  
			d[start_pos]=11;
			value=127-value;
		}else{
			d[start_pos]=110;
			value=value-127;//    0  127,  
		}			
	}
	
	if(length==2){
		d[start_pos+minus]=value/10;
		d[start_pos+1+minus]=value%10;
	}else{
		d[start_pos+minus]=value/100;
		d[start_pos+1+minus]=value/10-d[start_pos+minus]*10;
		d[start_pos+2+minus]=value%10;
	}
	points[0]=0;
	points[1]=0;
	points[2]=0;
	if(dot_position!=5){
		points[dot_position]=1;
	}
}



void setColor(uint8_t R,uint8_t G,uint8_t B){
	OCR0=R;
	OCR1BL=G;
	OCR1AL=B;
}

//    
void sendPreparedData(){
	int1_off;
	rfTxMode();
	SendPackage(data1);
	rfRxMode();
	FIFOReset();
	int1_on;	
}

int main(void){
	init();

	wdt_enable(WDTO_1S);
	uint8_t time_correction_applied=0;	
	for(;;){
		wdt_reset();
		if(needToSendPackage){
			data1[7]=GetCRC(data1);
			sendPreparedData();
			needToSendPackage=0;
		}
		if(interrupt_triggered){
			
			if(backToMenuCounter>9){
				if(currentMenuItem>0){
					currentMenuItem=0;
					current_mode=MODE_MENU;
				}
				backToMenuCounter=0;
			}
			backToMenuCounter++;
			
			if(current_mode!=MODE_EDITING){//  
				rtc_get_time(&data[10],&data[11],&data[12]);
				rtc_get_date(&data[13],&data[14],&data[15]);
			}else{//     
				if(currentMenuItem>9&&currentMenuItem<16){
					rtc_set_time(data[10],data[11],data[12]);
					rtc_set_date(data[13],data[14],data[15]);
				}				
			}
			
			//  
			uint8_t day_of_week=0;
			rtc_get_byte(&day_of_week,3);
			rtc_get_byte(&time_correction_applied,49);
			if(day_of_week==5){//  ( - 5-,         ,   )
				if(!time_correction_applied){//   
					if(data[11]==10){// 10     (     :) )
						if(data[12]==0){//   -    ,     
							uint8_t correction_value=data[16];
							//     , ,      ,        
							if((correction_value-127)<0){
								correction_value=127-correction_value;
								data[11]=data[11]-correction_value/60-1;
								data[12]=60-correction_value%60;
							}else{
								data[11]+=(correction_value-127)/60;
								data[12]+=(correction_value-127)%60;
							}
							rtc_set_time(data[10],data[11],data[12]);
							rtc_set_byte(1,49);//     ,    ,      
						}
					}
				}
			}else{
				rtc_set_byte(0,49);// ,     
			}

			
			//  (/)
			uint8_t flag=0;
			if(data[10]>data[3]&&data[10]<data[7]){
				flag=1;
			}else if(data[10]==data[3]){
				if(data[11]>data[4]){
					flag=1;
				}
			}else if(data[10]==data[7]){
				if(data[11]<data[8]){
					flag=1;
				}
			}
			if(flag){
				currentDayTime=DAY_MODE;
			}else{
				currentDayTime=NIGHT_MODE;
			}

			// 
			pushTemp(therm_read_temperature());
			
			//  / 
			if(currentTemp-data[(currentDayTime==NIGHT_MODE)?6:2]>target_temp[0+(currentDayTime==NIGHT_MODE)]){
				heater_off;
				points[3]=0;
			}else if(currentTemp+data[(currentDayTime==NIGHT_MODE)?6:2]<target_temp[0+(currentDayTime==NIGHT_MODE)]){
				heater_on;
				points[3]=1;//        ( )
			}		
			
			
			//     
			led_steps=data[23]*60;
			if(led_steps>0){
				if(currentDayTime==DAY_MODE){
					led_current_step=(uint16_t)(data[10]-data[3])*3600+(uint16_t)(data[11]-data[4])*60+data[12];
				}else{
					if(data[10]<data[3]){//   ,     (                 )
						led_current_step=led_steps;
					}else{//   ,      +          
						led_current_step=(uint16_t)(data[10]-data[7])*3600+(uint16_t)(data[11]-data[8])*60+data[12];
					}					
				}
				if(led_current_step>led_steps){
					led_current_step=led_steps;
				}
				if(currentDayTime==NIGHT_MODE){
					currentColor[0]=data[17]+((int32_t)(data[20]-data[17])*led_current_step/led_steps);
					currentColor[1]=data[18]+((int32_t)(data[21]-data[18])*led_current_step/led_steps);
					currentColor[2]=data[19]+((int32_t)(data[22]-data[19])*led_current_step/led_steps);
				}else{
					currentColor[0]=data[20]+((int32_t)(data[17]-data[20])*led_current_step/led_steps);
					currentColor[1]=data[21]+((int32_t)(data[18]-data[21])*led_current_step/led_steps);
					currentColor[2]=data[22]+((int32_t)(data[19]-data[22])*led_current_step/led_steps);
				}			
			}else{
				if(currentDayTime==DAY_MODE){
					currentColor[0]=data[17];
					currentColor[1]=data[18];
					currentColor[2]=data[19];
				}else{
					currentColor[0]=data[20];
					currentColor[1]=data[21];
					currentColor[2]=data[22];
				}
			}			
			
			setColor(currentColor[0],currentColor[1],currentColor[2]);
			
			
			//      (      )
			updateDisplayInfo();
			
			//    ,   
			if(current_mode!=old_mode||currentTemp!=old_temp){
				data1[0]=currentTemp/100;
				data1[1]=currentTemp/10-data1[0]*10+0x30;
				data1[2]='.';
				data1[0]+=0x30;
				data1[3]=currentTemp%10+0x30;
				data1[4]=' ';
				data1[5]=(currentDayTime==NIGHT_MODE)?'N':'D';//  - D - /N-
				data1[6]=(points[3]==1)?'+':'-';//+,   , - -  
				data1[7]=GetCRC(data1);
				sendPreparedData();
				
			}	

			old_mode=current_mode;
			old_temp=currentTemp;
			interrupt_triggered=0;		
		}
	}		
	
}